using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using NestedHtmlWriter;
using MagSiteLib;

namespace MagSite
{
	/// <summary>
	/// WebForm1 ̊Tv̐łB
	/// </summary>
	public class DefaultPage : System.Web.UI.Page
	{
		private void writeMessageList( FullKeyword keyword, NhQuickDocument doc, ItemList itemList, 
			string messageTitle )
		{
			ItemID [] itemIDs = itemList.GetItemIDs();
			if( itemIDs.Length > 0 )
			{
				using( NhHx h2 = doc.B.CreateHx(2) )
				{
					h2.WriteText(messageTitle);
				}
				Array.Sort(itemIDs);
				using( NhTable table = doc.B.CreateTable() )
				{
					for( int i=0; i<Math.Min(5,itemIDs.Length); i++ )
					{
						using( NhTr tr = table.CreateTr() )
						{
							Item item = itemList.GetItem( itemIDs[itemIDs.Length-i-1] );
							using( NhTdInline tdDate = tr.CreateTdInline() )
							{
								tdDate.WriteClassAttr("date");
								tdDate.WriteText(item.Date.ToString("yyyyNMMdd"));
							}
							using( NhTdInline tdMessage = tr.CreateTdInline() )
							{
								if( item.BodyType != EBodyType.PlaneText )
								{
									tdMessage.WriteRawString(item.Body);
								}
								else
								{
									tdMessage.WriteText(item.Body);
								}
							}
						}
					}
				}
			}
		}

		private const int newContentsItems = 9;
		private readonly TimeSpan TwoDaySpan = TimeSpan.FromDays(2);
		private void writeNewContentsList( FullKeyword keyword, NhQuickDocument doc )
		{
			ItemID [] list;
			if( keyword.IsNoKeyword() )
			{
				list = ItemListManager.ContentList.GetItemIDs();
			}
			else
			{
				list = ItemListManager.ContentList.GetItemIDsIncludingSubKeywords(keyword);
			}
			if( list.Length == 0 ) return;
			Array.Sort(list);

			SectionDef def = SectionDefManager.GetSectionDef(keyword);
			doc.B.WriteHxText(2,def.NewContentsTitle);

			int p = list.Length-1;
			int generatedItems = 0;
			DateTime limitDate = DateTime.Today.Subtract(TwoDaySpan);
			using( NhTable table = doc.B.CreateTable() )
			{
				while( true )
				{
					if( p < 0 ) break;
					DateTime itemDateTime = list[p].GetDateTime();
					Item item = ItemListManager.ContentList.GetItem(list[p]);
					if( itemDateTime < limitDate
						&& generatedItems > newContentsItems ) break;

					int count = ContentAccessRanking.GetTotalAccessRanking(keyword).GetCount(list[p]);
					WriteLib.WriteContentListTr(table,-1,itemDateTime,DateTime.MinValue,
						item.Keyword,FileNames.Default,list[p],count,"total",string.Empty);

					generatedItems++;
					p--;
				}
			}
		}

		private const int topContentsItems = 9;
		private void writeTopContentsList( FullKeyword keyword, NhQuickDocument doc )
		{
			DateTime today = DateTime.Today;
			PointTargetPair [] topPairs = ContentAccessRanking.GetAccessRanking(keyword,today).GetTopList(topContentsItems);
			if( topPairs.LongLength == 0 ) return;

			SectionDef def = SectionDefManager.GetSectionDef(keyword);
			doc.B.WriteHxText(2,def.TopContentsTitle);

			using( NhTable table = doc.B.CreateTable() )
			{
				for( int i=0; i<topPairs.Length; i++ )
				{
					ItemID itemID = (ItemID)topPairs[i].Target;
					Item item = item = ItemListManager.ContentList.GetItem(itemID);
					int count = topPairs[i].Count;
					//count = ContentAccessRanking.GetAccessRanking(keyword,today).GetCount(itemID);
					WriteLib.WriteContentListTr(table,i+1,itemID.GetDateOnly(),DateTime.MinValue,
						item.Keyword,FileNames.Default,itemID,count,"3days",string.Empty);
				}
			}
		}

		private const int topKeywordItems = 9;
		private void writeTopKeywordsList( FullKeyword keyword, NhQuickDocument doc )
		{
			DateTime today = DateTime.Today;
			PointTargetPair [] topPairs = KeywordAccessRanking.GetAccessRanking(keyword,today).GetTopListForKeywordRanking(topContentsItems);
			if( topPairs.LongLength == 0 ) return;

			SectionDef def = SectionDefManager.GetSectionDef(keyword);
			doc.B.WriteHxText(2,def.TopKeywordsTitle);

			using( NhTable table = doc.B.CreateTable() )
			{
				for( int i=0; i<topPairs.Length; i++ )
				{
					WriteLib.WriteContentListTr(table,i+1,DateTime.MinValue,DateTime.MinValue,
						(FullKeyword)topPairs[i].Target,
						FileNames.Default,	ItemID.NoItem,topPairs[i].Count,"3days",string.Empty);
				}
			}
		}

		private void writeLink( FullKeyword keyword, NhQuickDocument doc )
		{
			Links links = LinkManager.GetLinks( keyword );
			if( links.Length > 0 )
			{
				SectionDef def = SectionDefManager.GetSectionDef(keyword);
				doc.B.WriteHxText(2,def.LinkTitle);

				using( NhP linkPara = doc.B.CreateP() )
				{
					foreach( LinkItem item in links )
					{
						linkPara.WriteText("k");
						linkPara.WriteAText( item.Url, item.Description );
						linkPara.WriteText("l");
					}
				}
			}
		}

		private void writeAccessCount( FullKeyword keyword, NhP parentElement )
		{
			DateTime today = DateTime.Today;
			int todayCount = KeywordAccessRanking.GetAccessRanking(FullKeyword.NoKeyword,today).GetCount(keyword);
			int total = KeywordAccessRanking.GetTotalAccessRanking(FullKeyword.NoKeyword).GetCount(keyword);

			parentElement.WriteText( "L[[h" );
			WriteLib.WriteKeyword( parentElement, keyword, FileNames.Default, string.Empty );
			parentElement.WriteText( "ւ̃ANZXJEg: " );
			parentElement.WriteText( todayCount.ToString() );
			parentElement.WriteText( " (ߋ3)/ " );
			parentElement.WriteText( total.ToString() );
			parentElement.WriteText( " (݌v)" );
		}

		private void Page_Load(object sender, System.EventArgs e)
		{
			FullKeyword keyword = URIQueryParser.ParseKeyword(Request);
			SectionDef def = SectionDefManager.GetSectionDef(keyword);

			string title = def.Title;
			string subTitle = "";
			string pageTitle = title;
			if( !keyword.IsRootKeyword() )
			{
				subTitle = keyword.Keywords[keyword.Keywords.Length-1];
				pageTitle = title + " " + "y" + subTitle + "z";
			}

			using( NhQuickDocument doc = new MyOutputDocument( Response.Output, pageTitle, keyword ) )
			{
				using( NhTable pageTopBox = doc.B.CreateTable() )
				{
					pageTopBox.WriteClassAttr("boxed");
					using( NhTr pageTopTr = pageTopBox.CreateTr() )
					{
						using( NhTdBlock pageTopPicTd = pageTopTr.CreateTdBlock() )
						{
							using( NhP p = pageTopPicTd.CreateP() )
							{
								p.WriteClassAttr("titlePicture");
								//p.WriteText("A");
								using( NhImg pageTopPicImg = p.CreateImg() )
								{
									pageTopPicImg.WriteAttribute("src",def.TitlePictureURI);
								}
								//p.WriteText("A");
							}
						}
						using( NhTdBlock pageTopTextTd = pageTopTr.CreateTdBlock() )
						{
							using( NhDiv titleBox = pageTopTextTd.CreateDiv() )
							{
								titleBox.WriteClassAttr("mainTitleText");
								if( subTitle.Length > 0 )
								{
									using( NhP siteTitleP = titleBox.CreateP() )
									{
										siteTitleP.WriteClassAttr("mainTitleSupply");
										siteTitleP.WriteText(title);
									}
									using( NhP siteKeywordP = titleBox.CreateP() )
									{
										//siteKeywordP.WriteText("y");
										WriteLib.WriteKeyword(siteKeywordP,keyword,FileNames.Default,string.Empty);
										//siteKeywordP.WriteText("z");
									}
									//WriteLib.WriteHorizontalLine(titleBox);
									using( NhHx h1 = titleBox.CreateHx(1) )
									{
										h1.WriteClassAttr("maintitle");
										h1.WriteText(subTitle);
									}
								}
								else
								{
									using( NhHx h1 = titleBox.CreateHx(1) )
									{
										h1.WriteClassAttr("maintitlealone");
										h1.WriteText(title);
									}
								}
							}
						}
					}
				}

				if( def.HeadTextType != EBodyType.PlaneText )
				{
					doc.B.WriteRawString(def.HeadText);
				}
				else
				{
					WriteLib.WritePlainText(doc.B,def.HeadText);
				}

				writeMessageList( keyword, doc, ItemListManager.SystemMessageList, def.SystemMessageTitle );
#if USE_SHORT_MESSAGE
				writeMessageList( keyword, doc, ItemListManager.ShortMessageList, def.ShortMessageTitle );
#endif

				//doc.B.WriteHxText(2,def.ContentsTitle);
				//doc.B.WriteRawString(def.ContentsHeadText);

				writeNewContentsList(keyword,doc);

				writeTopContentsList(keyword,doc);

				writeTopKeywordsList(keyword,doc);

				WriteLib.WriteChildKeywordsList(keyword,doc,def.ChildKeywordsTitle,FileNames.Default,string.Empty);

				writeLink(keyword,doc);

				if( def.FootTextType != EBodyType.PlaneText )
				{
					doc.B.WriteRawString(def.FootText);
				}
				else
				{
					WriteLib.WritePlainText(doc.B,def.FootText);
				}

				WriteLib.WriteHorizontalLine( doc.B );

				using( NhP fixedFooterPara1 = doc.B.CreateP() )
				{
					writeAccessCount(keyword,fixedFooterPara1);
				}
				WriteLib.WriteNormalFooter( doc.B, keyword );
			}
		}

		#region Web tH[ fUCiŐꂽR[h 
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: ̌ĂяóAASP.NET Web tH[ fUCiŕKvłB
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// fUCi T|[gɕKvȃ\bhłB̃\bh̓e
		/// R[h GfB^ŕύXȂłB
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
